@@ -20,18 +20,16 @@ import mock
 from oslo_serialization import jsonutils
 
 from nova.compute import rpcapi as compute_rpcapi
-import nova.conf
 from nova import context
 from nova import exception
 from nova.objects import block_device as objects_block_dev
-from nova.objects import migrate_data as migrate_data_obj
 from nova.objects import migration as migration_obj
+from nova.objects import service as service_obj
 from nova import test
 from nova.tests.unit import fake_block_device
 from nova.tests.unit import fake_flavor
 from nova.tests.unit import fake_instance
-
-CONF = nova.conf.CONF
+from nova.tests import uuidsentinel as uuids
 
 
 class ComputeRpcAPITestCase(test.NoDBTestCase):
@@ -91,7 +89,9 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
         self.flags(compute='auto', group='upgrade_levels')
         compute_rpcapi.LAST_VERSION = None
         rpcapi = compute_rpcapi.ComputeAPI()
-        self.assertEqual('4.11', rpcapi.router.version_cap)
+        history = service_obj.SERVICE_VERSION_HISTORY
+        current_version = history[service_obj.SERVICE_VERSION]['compute_rpc']
+        self.assertEqual(current_version, rpcapi.router.version_cap)
         mock_get_min.assert_called_once_with(mock.ANY, 'nova-compute')
         self.assertIsNone(compute_rpcapi.LAST_VERSION)
 
@@ -111,16 +111,25 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
 
         rpcapi = kwargs.pop('rpcapi_class', compute_rpcapi.ComputeAPI)()
         self.assertIsNotNone(rpcapi.router)
-        self.assertEqual(rpcapi.router.target.topic, CONF.compute_topic)
+        self.assertEqual(rpcapi.router.target.topic,
+                         compute_rpcapi.RPC_TOPIC)
 
         # This test wants to run the real prepare function, so must use
         # a real client object
-        default_client = rpcapi.router.clients['default'].client
+        default_client = rpcapi.router.default_client
 
         orig_prepare = default_client.prepare
         base_version = rpcapi.router.target.version
         expected_version = kwargs.pop('version', base_version)
 
+        prepare_extra_kwargs = {}
+        cm_timeout = kwargs.pop('call_monitor_timeout', None)
+        timeout = kwargs.pop('timeout', None)
+        if cm_timeout:
+            prepare_extra_kwargs['call_monitor_timeout'] = cm_timeout
+        if timeout:
+            prepare_extra_kwargs['timeout'] = timeout
+
         expected_kwargs = kwargs.copy()
         if expected_args:
             expected_kwargs.update(expected_args)
@@ -135,10 +144,14 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
                 kwargs['cast'] = False
             else:
                 kwargs['do_cast'] = False
+        elif method == 'prep_resize' and 'migration' not in expected_args:
+            del expected_kwargs['migration']
         if 'host' in kwargs:
             host = kwargs['host']
         elif 'instances' in kwargs:
             host = kwargs['instances'][0]['host']
+        elif 'destination' in kwargs:
+            host = expected_kwargs.pop('destination')
         else:
             host = kwargs['instance']['host']
 
@@ -167,7 +180,8 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
             self.assertEqual(retval, rpc_mock.return_value)
 
             prepare_mock.assert_called_once_with(version=expected_version,
-                                                 server=host)
+                                                 server=host,
+                                                 **prepare_extra_kwargs)
             rpc_mock.assert_called_once_with(ctxt, method, **expected_kwargs)
 
     def test_add_aggregate_host(self):
@@ -178,89 +192,62 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
     def test_add_fixed_ip_to_instance(self):
         self._test_compute_api('add_fixed_ip_to_instance', 'cast',
                 instance=self.fake_instance_obj, network_id='id',
-                version='4.0')
+                version='5.0')
 
     def test_attach_interface(self):
         self._test_compute_api('attach_interface', 'call',
                 instance=self.fake_instance_obj, network_id='id',
-                port_id='id2', version='4.0', requested_ip='192.168.1.50')
+                port_id='id2', version='5.0', requested_ip='192.168.1.50',
+                tag='foo')
 
     def test_attach_volume(self):
         self._test_compute_api('attach_volume', 'cast',
                 instance=self.fake_instance_obj, bdm=self.fake_volume_bdm,
-                version='4.0')
+                version='5.0')
 
     def test_change_instance_metadata(self):
         self._test_compute_api('change_instance_metadata', 'cast',
-                instance=self.fake_instance_obj, diff={}, version='4.0')
+                instance=self.fake_instance_obj, diff={}, version='5.0')
 
     def test_check_instance_shared_storage(self):
         self._test_compute_api('check_instance_shared_storage', 'call',
                 instance=self.fake_instance_obj, data='foo',
-                version='4.0')
+                version='5.0')
 
     def test_confirm_resize_cast(self):
         self._test_compute_api('confirm_resize', 'cast',
                 instance=self.fake_instance_obj, migration={'id': 'foo'},
-                host='host', reservations=list('fake_res'))
+                host='host')
 
     def test_confirm_resize_call(self):
         self._test_compute_api('confirm_resize', 'call',
                 instance=self.fake_instance_obj, migration={'id': 'foo'},
-                host='host', reservations=list('fake_res'))
+                host='host')
 
     def test_detach_interface(self):
         self._test_compute_api('detach_interface', 'cast',
-                version='4.0', instance=self.fake_instance_obj,
+                version='5.0', instance=self.fake_instance_obj,
                 port_id='fake_id')
 
     def test_detach_volume(self):
         self._test_compute_api('detach_volume', 'cast',
                 instance=self.fake_instance_obj, volume_id='id',
-                attachment_id='fake_id', version='4.7')
-
-    def test_detach_volume_no_attachment_id(self):
-        ctxt = context.RequestContext('fake_user', 'fake_project')
-        instance = self.fake_instance_obj
-        rpcapi = compute_rpcapi.ComputeAPI()
-        cast_mock = mock.Mock()
-        cctxt_mock = mock.Mock(cast=cast_mock)
-        rpcapi.router.by_instance = mock.Mock()
-        mock_client = mock.Mock()
-        rpcapi.router.by_instance.return_value = mock_client
-        with test.nested(
-            mock.patch.object(mock_client, 'can_send_version',
-                              return_value=False),
-            mock.patch.object(mock_client, 'prepare',
-                              return_value=cctxt_mock)
-        ) as (
-            can_send_mock, prepare_mock
-        ):
-            rpcapi.detach_volume(ctxt, instance=instance,
-                                 volume_id='id', attachment_id='fake_id')
-        # assert our mocks were called as expected
-        can_send_mock.assert_called_once_with('4.7')
-        prepare_mock.assert_called_once_with(server=instance['host'],
-                                             version='4.0')
-        cast_mock.assert_called_once_with(ctxt, 'detach_volume',
-                                          instance=instance,
-                                          volume_id='id')
+                attachment_id='fake_id', version='5.0')
 
     def test_finish_resize(self):
         self._test_compute_api('finish_resize', 'cast',
                 instance=self.fake_instance_obj, migration={'id': 'foo'},
-                image='image', disk_info='disk_info', host='host',
-                reservations=list('fake_res'))
+                image='image', disk_info='disk_info', host='host')
 
     def test_finish_revert_resize(self):
         self._test_compute_api('finish_revert_resize', 'cast',
                 instance=self.fake_instance_obj, migration={'id': 'fake_id'},
-                host='host', reservations=list('fake_res'))
+                host='host')
 
     def test_get_console_output(self):
         self._test_compute_api('get_console_output', 'call',
                 instance=self.fake_instance_obj, tail_length='tl',
-                version='4.0')
+                version='5.0')
 
     def test_get_console_pool_info(self):
         self._test_compute_api('get_console_pool_info', 'call',
@@ -271,43 +258,43 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
 
     def test_get_diagnostics(self):
         self._test_compute_api('get_diagnostics', 'call',
-                instance=self.fake_instance_obj, version='4.0')
+                instance=self.fake_instance_obj, version='5.0')
 
     def test_get_instance_diagnostics(self):
         expected_args = {'instance': self.fake_instance_obj}
         self._test_compute_api('get_instance_diagnostics', 'call',
                 expected_args, instance=self.fake_instance_obj,
-                version='4.13')
+                version='5.0')
 
     def test_get_vnc_console(self):
         self._test_compute_api('get_vnc_console', 'call',
                 instance=self.fake_instance_obj, console_type='type',
-                version='4.0')
+                version='5.0')
 
     def test_get_spice_console(self):
         self._test_compute_api('get_spice_console', 'call',
                 instance=self.fake_instance_obj, console_type='type',
-                version='4.0')
+                version='5.0')
 
     def test_get_rdp_console(self):
         self._test_compute_api('get_rdp_console', 'call',
                 instance=self.fake_instance_obj, console_type='type',
-                version='4.0')
+                version='5.0')
 
     def test_get_serial_console(self):
         self._test_compute_api('get_serial_console', 'call',
                 instance=self.fake_instance_obj, console_type='serial',
-                version='4.0')
+                version='5.0')
 
     def test_get_mks_console(self):
         self._test_compute_api('get_mks_console', 'call',
                 instance=self.fake_instance_obj, console_type='webmks',
-                version='4.3')
+                version='5.0')
 
     def test_validate_console_port(self):
         self._test_compute_api('validate_console_port', 'call',
                 instance=self.fake_instance_obj, port="5900",
-                console_type="novnc", version='4.0')
+                console_type="novnc", version='5.0')
 
     def test_host_maintenance_mode(self):
         self._test_compute_api('host_maintenance_mode', 'call',
@@ -326,18 +313,18 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
                 instance=self.fake_instance_obj, dest='dest',
                 block_migration='blockity_block', host='tsoh',
                 migration='migration',
-                migrate_data={}, version='4.8')
+                migrate_data={}, version='5.0')
 
     def test_live_migration_force_complete(self):
         migration = migration_obj.Migration()
         migration.id = 1
         migration.source_compute = 'fake'
         ctxt = context.RequestContext('fake_user', 'fake_project')
-        version = '4.12'
+        version = '5.0'
         rpcapi = compute_rpcapi.ComputeAPI()
-        rpcapi.router.by_host = mock.Mock()
+        rpcapi.router.client = mock.Mock()
         mock_client = mock.MagicMock()
-        rpcapi.router.by_host.return_value = mock_client
+        rpcapi.router.client.return_value = mock_client
         mock_client.can_send_version.return_value = True
         mock_cctx = mock.MagicMock()
         mock_client.prepare.return_value = mock_cctx
@@ -349,37 +336,15 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
                                           'live_migration_force_complete',
                                           instance=self.fake_instance_obj)
 
-    def test_live_migration_force_complete_backward_compatibility(self):
-        migration = migration_obj.Migration()
-        migration.id = 1
-        migration.source_compute = 'fake'
-        version = '4.9'
-        ctxt = context.RequestContext('fake_user', 'fake_project')
-        rpcapi = compute_rpcapi.ComputeAPI()
-        rpcapi.router.by_host = mock.Mock()
-        mock_client = mock.MagicMock()
-        rpcapi.router.by_host.return_value = mock_client
-        mock_client.can_send_version.return_value = False
-        mock_cctx = mock.MagicMock()
-        mock_client.prepare.return_value = mock_cctx
-        rpcapi.live_migration_force_complete(ctxt, self.fake_instance_obj,
-                                             migration)
-        mock_client.prepare.assert_called_with(server=migration.source_compute,
-                                               version=version)
-        mock_cctx.cast.assert_called_with(ctxt,
-                                          'live_migration_force_complete',
-                                          instance=self.fake_instance_obj,
-                                          migration_id=migration.id)
-
     def test_live_migration_abort(self):
         self._test_compute_api('live_migration_abort', 'cast',
                 instance=self.fake_instance_obj,
-                migration_id='1', version='4.10')
+                migration_id='1', version='5.0')
 
     def test_post_live_migration_at_destination(self):
-        self._test_compute_api('post_live_migration_at_destination', 'cast',
+        self._test_compute_api('post_live_migration_at_destination', 'call',
                 instance=self.fake_instance_obj,
-                block_migration='block_migration', host='host', version='4.0')
+                block_migration='block_migration', host='host', version='5.0')
 
     def test_pause_instance(self):
         self._test_compute_api('pause_instance', 'cast',
@@ -387,43 +352,47 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
 
     def test_soft_delete_instance(self):
         self._test_compute_api('soft_delete_instance', 'cast',
-                instance=self.fake_instance_obj,
-                reservations=['uuid1', 'uuid2'])
+                instance=self.fake_instance_obj)
 
     def test_swap_volume(self):
         self._test_compute_api('swap_volume', 'cast',
                 instance=self.fake_instance_obj, old_volume_id='oldid',
-                new_volume_id='newid')
+                new_volume_id='newid', new_attachment_id=uuids.attachment_id,
+                version='5.0')
 
     def test_restore_instance(self):
         self._test_compute_api('restore_instance', 'cast',
-                instance=self.fake_instance_obj, version='4.0')
+                instance=self.fake_instance_obj, version='5.0')
 
     def test_pre_live_migration(self):
+        self.flags(long_rpc_timeout=1234)
         self._test_compute_api('pre_live_migration', 'call',
                 instance=self.fake_instance_obj,
                 block_migration='block_migration', disk='disk', host='host',
-                migrate_data=None, version='4.8')
+                migrate_data=None, version='5.0',
+                call_monitor_timeout=60, timeout=1234)
+
+    def test_check_can_live_migrate_destination(self):
+        self.flags(long_rpc_timeout=1234)
+        self._test_compute_api('check_can_live_migrate_destination', 'call',
+                               instance=self.fake_instance_obj,
+                               destination='dest',
+                               block_migration=False,
+                               disk_over_commit=False,
+                               version='5.0', call_monitor_timeout=60,
+                               timeout=1234)
 
     def test_prep_resize(self):
-        self._test_compute_api('prep_resize', 'cast',
-                instance=self.fake_instance_obj,
-                instance_type=self.fake_flavor_obj,
-                image='fake_image', host='host',
-                reservations=list('fake_res'),
-                request_spec='fake_spec',
-                filter_properties={'fakeprop': 'fakeval'},
-                node='node', clean_shutdown=True, version='4.1')
-        self.flags(compute='4.0', group='upgrade_levels')
-        expected_args = {'instance_type': self.fake_flavor}
+        expected_args = {'migration': 'migration'}
         self._test_compute_api('prep_resize', 'cast', expected_args,
                 instance=self.fake_instance_obj,
                 instance_type=self.fake_flavor_obj,
                 image='fake_image', host='host',
-                reservations=list('fake_res'),
                 request_spec='fake_spec',
                 filter_properties={'fakeprop': 'fakeval'},
-                node='node', clean_shutdown=True, version='4.0')
+                migration='migration',
+                node='node', clean_shutdown=True, host_list=None,
+                version='5.0')
 
     def test_reboot_instance(self):
         self.maxDiff = None
@@ -438,28 +407,20 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
                 bdms=[], instance=self.fake_instance_obj, host='new_host',
                 orig_sys_metadata=None, recreate=True, on_shared_storage=True,
                 preserve_ephemeral=True, migration=None, node=None,
-                limits=None, version='4.5')
-
-    def test_rebuild_instance_downgrade(self):
-        self.flags(group='upgrade_levels', compute='4.0')
-        self._test_compute_api('rebuild_instance', 'cast', new_pass='None',
-                injected_files='None', image_ref='None', orig_image_ref='None',
-                bdms=[], instance=self.fake_instance_obj, host='new_host',
-                orig_sys_metadata=None, recreate=True, on_shared_storage=True,
-                preserve_ephemeral=True, version='4.0')
+                limits=None, request_spec=None, version='5.0')
 
     def test_reserve_block_device_name(self):
         self._test_compute_api('reserve_block_device_name', 'call',
                 instance=self.fake_instance_obj, device='device',
                 volume_id='id', disk_bus='ide', device_type='cdrom',
-                version='4.0',
+                tag='foo', multiattach=True, version='5.0',
                 _return_value=objects_block_dev.BlockDeviceMapping())
 
     def test_refresh_instance_security_rules(self):
         expected_args = {'instance': self.fake_instance_obj}
         self._test_compute_api('refresh_instance_security_rules', 'cast',
                 expected_args, host='fake_host',
-                instance=self.fake_instance_obj, version='4.4')
+                instance=self.fake_instance_obj, version='5.0')
 
     def test_remove_aggregate_host(self):
         self._test_compute_api('remove_aggregate_host', 'cast',
@@ -469,18 +430,18 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
     def test_remove_fixed_ip_from_instance(self):
         self._test_compute_api('remove_fixed_ip_from_instance', 'cast',
                 instance=self.fake_instance_obj, address='addr',
-                version='4.0')
+                version='5.0')
 
     def test_remove_volume_connection(self):
         self._test_compute_api('remove_volume_connection', 'call',
                 instance=self.fake_instance_obj, volume_id='id', host='host',
-                version='4.0')
+                version='5.0')
 
     def test_rescue_instance(self):
         self._test_compute_api('rescue_instance', 'cast',
             instance=self.fake_instance_obj, rescue_password='pw',
             rescue_image_ref='fake_image_ref',
-            clean_shutdown=True, version='4.0')
+            clean_shutdown=True, version='5.0')
 
     def test_reset_network(self):
         self._test_compute_api('reset_network', 'cast',
@@ -490,15 +451,7 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
         self._test_compute_api('resize_instance', 'cast',
                 instance=self.fake_instance_obj, migration={'id': 'fake_id'},
                 image='image', instance_type=self.fake_flavor_obj,
-                reservations=list('fake_res'),
-                clean_shutdown=True, version='4.1')
-        self.flags(compute='4.0', group='upgrade_levels')
-        expected_args = {'instance_type': self.fake_flavor}
-        self._test_compute_api('resize_instance', 'cast', expected_args,
-                instance=self.fake_instance_obj, migration={'id': 'fake_id'},
-                image='image', instance_type=self.fake_flavor_obj,
-                reservations=list('fake_res'),
-                clean_shutdown=True, version='4.0')
+                clean_shutdown=True, version='5.0')
 
     def test_resume_instance(self):
         self._test_compute_api('resume_instance', 'cast',
@@ -507,12 +460,12 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
     def test_revert_resize(self):
         self._test_compute_api('revert_resize', 'cast',
                 instance=self.fake_instance_obj, migration={'id': 'fake_id'},
-                host='host', reservations=list('fake_res'))
+                host='host')
 
     def test_set_admin_password(self):
         self._test_compute_api('set_admin_password', 'call',
                 instance=self.fake_instance_obj, new_pass='pw',
-                version='4.0')
+                version='5.0')
 
     def test_set_host_enabled(self):
         self._test_compute_api('set_host_enabled', 'call',
@@ -537,12 +490,12 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
     def test_stop_instance_cast(self):
         self._test_compute_api('stop_instance', 'cast',
                 instance=self.fake_instance_obj,
-                clean_shutdown=True, version='4.0')
+                clean_shutdown=True, version='5.0')
 
     def test_stop_instance_call(self):
         self._test_compute_api('stop_instance', 'call',
                 instance=self.fake_instance_obj,
-                clean_shutdown=True, version='4.0')
+                clean_shutdown=True, version='5.0')
 
     def test_suspend_instance(self):
         self._test_compute_api('suspend_instance', 'cast',
@@ -551,7 +504,7 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
     def test_terminate_instance(self):
         self._test_compute_api('terminate_instance', 'cast',
                 instance=self.fake_instance_obj, bdms=[],
-                reservations=['uuid1', 'uuid2'], version='4.0')
+                version='5.0')
 
     def test_unpause_instance(self):
         self._test_compute_api('unpause_instance', 'cast',
@@ -559,39 +512,39 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
 
     def test_unrescue_instance(self):
         self._test_compute_api('unrescue_instance', 'cast',
-                instance=self.fake_instance_obj, version='4.0')
+                instance=self.fake_instance_obj, version='5.0')
 
     def test_shelve_instance(self):
         self._test_compute_api('shelve_instance', 'cast',
                 instance=self.fake_instance_obj, image_id='image_id',
-                clean_shutdown=True, version='4.0')
+                clean_shutdown=True, version='5.0')
 
     def test_shelve_offload_instance(self):
         self._test_compute_api('shelve_offload_instance', 'cast',
                 instance=self.fake_instance_obj,
-                clean_shutdown=True, version='4.0')
+                clean_shutdown=True, version='5.0')
 
     def test_unshelve_instance(self):
         self._test_compute_api('unshelve_instance', 'cast',
                 instance=self.fake_instance_obj, host='host', image='image',
                 filter_properties={'fakeprop': 'fakeval'}, node='node',
-                version='4.0')
+                version='5.0')
 
     def test_volume_snapshot_create(self):
         self._test_compute_api('volume_snapshot_create', 'cast',
                 instance=self.fake_instance_obj, volume_id='fake_id',
-                create_info={}, version='4.0')
+                create_info={}, version='5.0')
 
     def test_volume_snapshot_delete(self):
         self._test_compute_api('volume_snapshot_delete', 'cast',
                 instance=self.fake_instance_obj, volume_id='fake_id',
-                snapshot_id='fake_id2', delete_info={}, version='4.0')
+                snapshot_id='fake_id2', delete_info={}, version='5.0')
 
     def test_external_instance_event(self):
         self._test_compute_api('external_instance_event', 'cast',
                                instances=[self.fake_instance_obj],
                                events=['event'],
-                               version='4.0')
+                               version='5.0')
 
     def test_build_and_run_instance(self):
         self._test_compute_api('build_and_run_instance', 'cast',
@@ -600,35 +553,26 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
                 admin_password='passwd', injected_files=None,
                 requested_networks=['network1'], security_groups=None,
                 block_device_mapping=None, node='node', limits=[],
-                version='4.0')
+                host_list=None, version='5.0')
 
     def test_quiesce_instance(self):
         self._test_compute_api('quiesce_instance', 'call',
-                instance=self.fake_instance_obj, version='4.0')
+                instance=self.fake_instance_obj, version='5.0')
 
     def test_unquiesce_instance(self):
         self._test_compute_api('unquiesce_instance', 'cast',
-                instance=self.fake_instance_obj, mapping=None, version='4.0')
+                instance=self.fake_instance_obj, mapping=None, version='5.0')
 
     def test_trigger_crash_dump(self):
         self._test_compute_api('trigger_crash_dump', 'cast',
-                instance=self.fake_instance_obj, version='4.6')
-
-    def test_trigger_crash_dump_incompatible(self):
-        self.flags(compute='4.0', group='upgrade_levels')
-        self.assertRaises(exception.TriggerCrashDumpNotSupported,
-                          self._test_compute_api,
-                          'trigger_crash_dump', 'cast',
-                          instance=self.fake_instance_obj, version='4.6')
+                instance=self.fake_instance_obj, version='5.0')
 
     def _test_simple_call(self, method, inargs, callargs, callret,
                                calltype='call', can_send=False):
         rpc = compute_rpcapi.ComputeAPI()
         mock_client = mock.Mock()
-        rpc.router.by_instance = mock.Mock()
-        rpc.router.by_instance.return_value = mock_client
-        rpc.router.by_host = mock.Mock()
-        rpc.router.by_host.return_value = mock_client
+        rpc.router.client = mock.Mock()
+        rpc.router.client.return_value = mock_client
 
         @mock.patch.object(compute_rpcapi, '_compute_host')
         def _test(mock_ch):
@@ -638,128 +582,18 @@ class ComputeRpcAPITestCase(test.NoDBTestCase):
             ctxt = context.RequestContext()
             result = getattr(rpc, method)(ctxt, **inargs)
             call.assert_called_once_with(ctxt, method, **callargs)
-            # Get the target of the prepare call: prepare(server=<target>, ...)
-            prepare_target = mock_client.prepare.call_args[1]['server']
-            # If _compute_host(None, instance) was called, then by_instance
-            # should have been called with the instance. Otherwise by_host
-            # should have been called with the same host as the prepare target.
-            if mock_ch.called and mock_ch.call_args[0][0] is None:
-                instance = mock_ch.call_args[0][1]
-                rpc.router.by_instance.assert_called_once_with(ctxt, instance)
-                rpc.router.by_host.assert_not_called()
-            else:
-                rpc.router.by_host.assert_called_once_with(ctxt,
-                                                           prepare_target)
-                rpc.router.by_instance.assert_not_called()
+            rpc.router.client.assert_called_with(ctxt)
             return result
 
         return _test()
 
-    def test_check_can_live_migrate_source_converts_objects(self):
-        obj = migrate_data_obj.LiveMigrateData()
-        inst = self.fake_instance_obj
-        result = self._test_simple_call('check_can_live_migrate_source',
-                                        inargs={'instance': inst,
-                                                'dest_check_data': obj},
-                                        callargs={'instance': inst,
-                                                  'dest_check_data': {}},
-                                        callret=obj)
-        self.assertEqual(obj, result)
-        result = self._test_simple_call('check_can_live_migrate_source',
-                                        inargs={'instance': inst,
-                                                'dest_check_data': obj},
-                                        callargs={'instance': inst,
-                                                  'dest_check_data': {}},
-                                        callret={'foo': 'bar'})
-        self.assertIsInstance(result, migrate_data_obj.LiveMigrateData)
-
-    @mock.patch('nova.objects.migrate_data.LiveMigrateData.'
-                'detect_implementation')
-    def test_check_can_live_migrate_destination_converts_dict(self,
-                                                              mock_det):
-        inst = self.fake_instance_obj
-        result = self._test_simple_call('check_can_live_migrate_destination',
-                                        inargs={'instance': inst,
-                                                'destination': 'bar',
-                                                'block_migration': False,
-                                                'disk_over_commit': False},
-                                        callargs={'instance': inst,
-                                                  'block_migration': False,
-                                                  'disk_over_commit': False},
-                                        callret={'foo': 'bar'})
-        self.assertEqual(mock_det.return_value, result)
-
-    def test_live_migration_converts_objects(self):
-        obj = migrate_data_obj.LiveMigrateData()
-        inst = self.fake_instance_obj
-        self._test_simple_call('live_migration',
-                               inargs={'instance': inst,
-                                       'dest': 'foo',
-                                       'block_migration': False,
-                                       'host': 'foo',
-                                       'migration': None,
-                                       'migrate_data': obj},
-                               callargs={'instance': inst,
-                                         'dest': 'foo',
-                                         'block_migration': False,
-                                         'migrate_data': {
-                                             'pre_live_migration_result': {}}},
-                               callret=None,
-                               calltype='cast')
-
-    @mock.patch('nova.objects.migrate_data.LiveMigrateData.from_legacy_dict')
-    def test_pre_live_migration_converts_objects(self, mock_fld):
-        obj = migrate_data_obj.LiveMigrateData()
-        inst = self.fake_instance_obj
-        result = self._test_simple_call('pre_live_migration',
-                                        inargs={'instance': inst,
-                                                'block_migration': False,
-                                                'disk': None,
-                                                'host': 'foo',
-                                                'migrate_data': obj},
-                                        callargs={'instance': inst,
-                                                  'block_migration': False,
-                                                  'disk': None,
-                                                  'migrate_data': {}},
-                                        callret=obj)
-        self.assertFalse(mock_fld.called)
-        self.assertEqual(obj, result)
-        result = self._test_simple_call('pre_live_migration',
-                                        inargs={'instance': inst,
-                                                'block_migration': False,
-                                                'disk': None,
-                                                'host': 'foo',
-                                                'migrate_data': obj},
-                                        callargs={'instance': inst,
-                                                  'block_migration': False,
-                                                  'disk': None,
-                                                  'migrate_data': {}},
-                                        callret={'foo': 'bar'})
-        mock_fld.assert_called_once_with(
-            {'pre_live_migration_result': {'foo': 'bar'}})
-        self.assertIsInstance(result, migrate_data_obj.LiveMigrateData)
-
-    def test_rollback_live_migration_at_destination_converts_objects(self):
-        obj = migrate_data_obj.LiveMigrateData()
-        inst = self.fake_instance_obj
-        method = 'rollback_live_migration_at_destination'
-        self._test_simple_call(method,
-                               inargs={'instance': inst,
-                                       'host': 'foo',
-                                       'destroy_disks': False,
-                                       'migrate_data': obj},
-                               callargs={'instance': inst,
-                                         'destroy_disks': False,
-                                         'migrate_data': {}},
-                               callret=None,
-                               calltype='cast')
-
-    def test_check_can_live_migrate_destination_old_compute(self):
-        self.flags(compute='4.10', group='upgrade_levels')
-        self.assertRaises(exception.LiveMigrationWithOldNovaNotSupported,
-                          self._test_compute_api,
-                          'check_can_live_migrate_destination', 'call',
-                          instance=self.fake_instance_obj,
-                          block_migration=None,
-                          destination='dest',
-                          disk_over_commit=None, version='4.11')
+    @mock.patch('nova.compute.rpcapi.LOG')
+    @mock.patch('nova.objects.Service.get_minimum_version')
+    def test_version_cap_no_computes_log_once(self, mock_minver, mock_log):
+        self.flags(compute='auto', group='upgrade_levels')
+        mock_minver.return_value = 0
+        compute_rpcapi.NO_COMPUTES_WARNING = False
+        compute_rpcapi.LAST_VERSION = None
+        compute_rpcapi.ComputeAPI()
+        compute_rpcapi.ComputeAPI()
+        self.assertEqual(1, mock_log.debug.call_count)
